# extract data from a trial folder
# put into a dict
# 12/18/2016

import os, csv
import numpy as np

def readcsv(file_name):
    # read csv file generated by our data collection system:
    # first line is the start time, following lines are the data

    # read start_time
    f = open(file_name, mode='r')
    reader = csv.reader(f, delimiter=',')
    for row in reader:
        assert len(row) == 1, 'first line should be a single value'
        start_time = int(row[0])
        break
    f.close()

    # read data
    data = np.genfromtxt(file_name, delimiter=',', skip_header=1)

    return (start_time, data)

def match_patriot_FT(data_patriot, data_FT):
    '''
    match mini40 data with patriot: for each time stamp of patriot, find the
    FT data that has the closest time stamp. 

    Returns patriot tracker 1 data and FT data
    '''
    seq_len = data_patriot.shape[0]
    t_PO = data_patriot[:, 0] # PO: position and orientation
    t_FT = data_FT[:, 0]
    t_FT_match_index = np.zeros(t_PO.shape, dtype=np.int32)
    for i, time in enumerate(t_PO):
        t_FT_match_index[i] = np.argmin(np.absolute(time - t_FT))
    data_FT_match = data_FT[t_FT_match_index, :]

    data_patriot_notime = data_patriot[:, 2:8] # 2:8 are tracker 1 data
    data_FT_notime = data_FT[t_FT_match_index, 1:] 

    
    return (data_patriot_notime, data_FT_notime) 



def extract_a_trial(path):
    '''
    Extract data from a trial folder and put into a dict.
    Input:
        absolute path
    Return:
        either a dict which contains all required data, or a list of missing
        data

    Required data:
    1. mini40_initial
    2. mini40_final
    3. mini40
    4. patriot
    
    '''

    files = os.listdir(path)
    
    # check if required files exist
    has_data = []
    
    for f in files:
        f_abs = os.path.join(path, f)
        # extract FT_initial
        if 'mini40_initial.csv' in f:
            data_FT_init = np.genfromtxt(f_abs)
            has_data.append('FT_init')
        
        # extract FT_final
        if 'mini40_final.csv' in f:
            data_FT_final = np.genfromtxt(f_abs)
            has_data.append('FT_final')
        
        # extract FT
        if 'mini40.csv' in f:
            start_time_FT, data_FT = readcsv(f_abs)
            has_data.append('FT')
        
        # extract patriot
        if 'patriot.csv' in f:
            start_time_patriot, data_patriot = readcsv(f_abs)
            has_data.append('patriot')

    # end of for loop

    expected_data = ['FT_init', 'FT_final', 'FT', 'patriot']
    missing_data = []
    for e in expected_data:
        if e not in has_data:
            missing_data.append(e)

    if len(missing_data) > 0:
        return missing_data

    # assert start time is the same
    assert (start_time_FT == start_time_patriot)

    # match time stamp of patriot and FT data, then only return data
    data_patriot_notime, data_FT_notime = match_patriot_FT(data_patriot, data_FT)

    data_dict = {'FT': data_FT_notime,
        'FT_init': data_FT_init,
        'FT_final': data_FT_final,
        'patriot': data_patriot_notime}


    return data_dict

def extract_pair_cup_cont(pair_path):
    # extract the trials of one pair of cup and container


    material_dict = {'beans': 1, 'water': 2, 'ice': 3} # add material when available
    cup_string = 'cup'
    container_string = 'container'
    cup_index_max = 6 # modify when necessary
    container_index_max = 10 # modify when necessary

    trial_dict_list = [] # list of trial_dicts

    if pair_path[-1] == '/':
        pair_path = pair_path[:-1]
    slash_index = pair_path.rfind('/')
    name_1 = pair_path[slash_index + 1:]


    # name_1 = pair_path

    print(pair_path)
    # name_1 should be similar to 'beans_cup_6_container_01'
    cup_location = name_1.find(cup_string) + 4 # 4 for 'cup_'
    cup_index = int(name_1[cup_location])
    container_location = name_1.find(container_string) + 10 # 10 for 'container_'
    container_index = int(name_1[container_location: container_location+2])
    material_end_location = name_1.find(cup_string) - 1 # 1 for the '_' before 'cup'
    material_string = name_1[:material_end_location]

    material_warning = material_string + ' not found in material_dict'
    assert material_string in material_dict, material_warning
    cup_warning = 'cup ' + str(cup_index) + ' undefined'
    assert cup_index in range(1, cup_index_max+1), cup_warning
    container_warning = 'container ' + str(container_index) + ' undefined'
    assert cup_index in range(1, cup_index_max+1), container_warning

    # print('material', material_dict[material_string], 'cup', cup_index, 'container', container_index)

    dir_2 = pair_path
    dirs_2 = os.listdir(dir_2)

    list_tmp = []
    for name_2 in dirs_2:
        # print(name_2)
        # name_2 can be either 'tx' or 'empty_FT.txt'
        name_2_abs = os.path.join(dir_2, name_2)
        if os.path.isfile(name_2_abs):
            # 'empty_FT.txt'
            data_FT_empty = np.genfromtxt(name_2_abs)

        if os.path.isdir(name_2_abs):
            # trial folder
            trial_return = extract_a_trial(name_2_abs)
            if isinstance(trial_return, list):
                print(name_2_abs, 'lacks', trial_return)
            else:
                list_tmp.append(trial_return)
    # end for loop

    for e in list_tmp:
        e['FT_empty'] = data_FT_empty
        e['cup'] = cup_index
        e['container'] = container_index
        e['material'] = material_dict[material_string]
    trial_dict_list += list_tmp





